// P32PredCat2x2.sce
// Prediction with a "square" discrete model f(z(t)|z(t-1),th)             
// ----------------------------------------------------------
[u,t,n]=file();                         // find working directory
chdir(dirname(n(2)));                   // set working directory
clear("u","t","n")                      // clear auxiliary data
exec("ScIntro.sce",-1),mode(0)          // intro to sesion                 

np=5;                              // prediction length
nd=500;                            // data sample length
I_est=2;                           // estimation: 1=yes, 2=no
I_pred=2;                          // prediction: 1=arg max, 2=expect.

T=[.99 .005 .005;                  // simulation model parameter
   .01  .98  .01
   .02 .01  .97];
   
// SIMULATION
z(1)=1;                            // initial output
for t=2:nd                         // data generation
  z(t)=(sum(rand(1,1,'unif')>(cumsum(T(z(t-1),:)))))+1;
end

// ESTIMATION AND PREDICTION
S=ones(T);                         // initial model statistics
vals=(1:max(size(T)))';            // values of output
for t=2:nd
  S(z(t-1),z(t))=S(z(t-1),z(t))+1; // statistics update
  // known or estimated parameters
  select I_est                     // estimation  1 = no, 2 = yes
  case 1 then
    Te=T;
  case 2 then
    Te=fnorm(S,2);                 // parameter point estimates
  end
  Tp=Te^np;                        // model for prediction
  // type of prediction point estimates
  select I_pred         
  case 1 then 
    [xxx yy]=max(Tp(z(t),:)); 
    yp(t+np)=yy;
  case 2 then 
    yp(t+np)=Tp(z(t),:)*vals;
  end
end

// RESULTS
s=2:nd;
set(scf(1),'position',[100 100 500 400])
plot(s,z(s),'o',s,yp(s),'.')
set(gca(),'data_bounds',[1 nd .9 3.1]);
title 'Discrete data and their prediction'
legend('data','prediction');
