// P62ConCatAd.sce
// ADAPTIVE CONTROL WITH CATEGORICAL MODEL
//   model f(zt(t)|ut(t),zt(t-1))
//   control with with init or without it
//   comparison of controlled and un- criterion
                 
[u,t,n]=file();                         // find working directory
chdir(dirname(n(2)));                   // set working directory
clear("u","t","n")                      // clear auxiliary data
exec("ScIntro.sce",-1),mode(0)          // intro to sesion                 

nd=200;             // number of steps 

I_est=1;            // estimation 1=yes, 0=no (known parameters)
I_typPar=2;         // choice of parameters
I_priEst=1;         // prior estimation 0=no, 1=yes (zeroing V is skipped)

Sim.Cz.ky=2;        // dimension of zt
Sim.Cz.ku=3;        // dimension of ut
Con.Cz.nh=5;        // control horizon
// Construction of criterion (big values are penalized)
Con.Cz.om=[1 0
           0 1
           1 0
           0 1
           1 0
           0 1];
J=Con.Cz.om;           

// Construction of parameters (set by hand)
ky=Sim.Cz.ky;
ku=Sim.Cz.ku;
nh=Con.Cz.nh;

select I_typPar
case 1, a=1:ky; b=[]; for i=1:ku, b=[b; (a'+i)]; end
        th=b*ones(1,ky)+ones(ky*ku,1)*a;
        thS=th./(sum(th,2)*ones(1,ky));
case 2, thS=fnorm([1 0;0 1;1 0;1 0;0 1;.1 0]+.1,2);
end
J1=0; J2=0;    // crit, for comparison: without and with control

// SIMULATION WITH RANDOM INPUT --------------------------------
V=ones(ky*ku,ky);
yy(1)=1;
for t=2:nd
  uu(t)=sum(rand(1,1,'ut')>cumsum([.3 .5 .2]))+1;  // input
  j=ky*(uu(t)-1)+yy(t-1);
  yy(t)=sum(rand(1,1,'ut')>cumsum(thS(j,:)))+1;    // output
  J1=J1+J(j,yy(t));      // evaluation of criterion J1

  V(j,yy(t))=V(j,yy(t))+1;
  thE=V./(sum(V,2)*ones(1,ky));    // prior estimation
end

// SIMULATION WITH CONTROL -------------------------------------
if I_priEst==0, 
  V=ones(ky*ku,ky); 
  thE=V./(sum(V,2)*ones(1,ky));
end
zt(1)=1; ut(2)=1; ut(1)=1;
if I_est==1, thC=thE; else thC=thS; end   // estimation or known pars
// dynamic loop
for t=2:nd 
  // simulation
  j=ky*(ut(t)-1)+zt(t-1);
  zt(t)=sum(rand(1,1,'ut')>cumsum(thC(j,:)))+1;
  J2=J2+J(j,zt(t));       // evaluation of criterion J1

  // estimation
  V(j,zt(t))=V(j,zt(t))+1;
  thE=V./(sum(V,2)*ones(1,ky));

  // optimization
  fiS=zeros(ku*ky,ky);
  for h=1:nh
    for i=1:ky
      fi=sum((fiS+J).*thS,2);      // function phi
      for i=1:ky
        ff=[];
        for j=1:ku
          ff=[ff fi(i+(j-1)*ky)];
        end      
        [xxx jc]=min(ff);  
        c(t,i)=jc;
        cf(1,i)=fi(i+(jc-1)*ky);
      end
      fiS=[];
      for k=1:ky*ku
        fiS=[fiS; cf];             // function phi_star
      end
    end  
  end
  // control (only one step is applied)
  ut(t+1)=c(t,zt(t));
end
Con.Cz.zt=zt;
Con.Cz.ut=ut;
Con.Cz.Jcon=J2;
Con.Cz.Jnocon=J1;

// show criteria values J1-without, J2-with control
disp(J1,'criterion without control')
disp(J2,'criterion with control') 
disp('Goal of the control: to avoid big values of both y and u')

if 1, s=(nd-49):nd;
else  s=1:50; end        // show end or beginning of task
set(scf(1),'position',[500 50 600 400])
set(gca(),'data_bounds',[min(s) max(s) .9 ky+.1])
plot(s,ut(s),'ko')
plot(s,zt(s),'bx','markersize',8)
plot(s,yy(s),'m.')
title('Adsptive control with categorical model')
legend('input','output','no control');
