//P7xMixRealIni1.sce
// Estimation of a mixture for classification (in T73)
// - data from Skoda car used
// - initialization with EXPERTLY PRE-CLASSIFIED DATA
[u,t,n]=file();                         // find working directory
chdir(dirname(n(2)));                   // set working directory
clear("u","t","n")                      // clear auxiliary data
exec("ScIntro.sce",-1),mode(0)          // intro to sesion                 

nd=600;                  // number of data
T=100;                   // number of measurements in sampling period

// =========================== DATA
load _data/dataKF.dat    // data load
dt=DataKF.dataOrig;

cc=dt(6,:); // gear and its discretization
c5=zeros(1,length(cc));
j=find((cc==-1)|(cc==0)); c5(j)=1;
j=find((cc>.5)&(cc<3.5)); c5(j)=2;
j=find(cc==4); c5(j)=3;
j=find(cc==5); c5(j)=4;
j=find(cc==6); c5(j)=5;

ct=c5(1:T:$);
Sim.nc=max(ct);
yt=[dt(2,1:T:$);              // speed
dt(4,1:T:$);                  // gas
dt(7,1:T:$);                  // moment
dt(8,1:T:$)/1000];            // revs
Sim.Cy.my=size(yt,1);
Sim.Cy.yt=yt;
Sim.Cy.T=T;                   // model period of sampling (in T0)
Sim.Cy.T0='5 ms';             // basic period of sampling
Sim.Cp.yt=ct;
nc=Sim.nc;
my=Sim.Cy.my;

// =========================== INITIALIZATION
// - from expertly pre-classified data: here from real sample yt and ct
printf(' - initialization ...\n')
dd=list();
for i=1:nc
  j=find(ct==i);            // known classes
  k=min(150,length(j));
  dd(i)=yt(:,j(1:k));       // corresponding data
  Est.Cy(i).th=mean(dd(i),2);
  nn=max(size(dd(i)));
  ed=dd(i)-mean(dd(i),2)*ones(1,nn);
  Est.Cy(i).cv=ed*ed'/nn+1e-6*eye(my,my);
  Ps=[Est.Cy(i).th;1];
  Est.Cy(i).V=Ps*Ps';
end
Est.ka=ones(1,nc);            // initial counter
Est.Cp.V=ones(1,nc);          // initial pointer statistics
Est.Cp.th=fnorm(rand(1,nc,'u')+1); // initial pointer parameter
w=fnorm(ones(1,nc));          // initial likelihood

// =========================== TIME LOOP
printf( ' - time loop ...\n')
ce=ones(1,nd);
printf(' '), tt=fix(nd/10); 
for t=(2:nd)
  if t/tt==fix(t/tt), printf('.'); end
  for j=1:nc
    [xxx,G(j)]=GaussN(yt(:,t),Est.Cy(j).th,Est.Cy(j).cv); // likelihoods
  end
  Lq=G-max(G);
  q=exp(Lq);

  ww=q'.*Est.Cp.th; w=ww/sum(ww);            // weights construction
  wt(:,t)=w';

  // update of data statistics
  Ps=[yt(:,t)' 1]';                          // ext. reg. vector
  for i=1:nc
    Est.Cy(i).V=Est.Cy(i).V+w(i)*Ps*Ps';     // information matrix 
    Est.ka(i)=Est.ka(i)+w(i);                // counter
    Est.Cp.V(i)=Est.Cp.V(i)+w(i);            // pointer stat.

    Vyy=Est.Cy(i).V(1:my,1:my);              // part Vyy (psi.psi')
    Vy=Est.Cy(i).V($,1:my);                  // part Vy  (y.psi)
    V1=Est.Cy(i).V($,$);                     // part V1  (y.y)
    Est.Cy(i).th=inv(V1+1e-8*eye(V1))*Vy;    // theta est.
    Est.Cy(i).tht(:,t)=Est.Cy(i).th';        //   remember
    if 0 // 0=all with initial covariances ( recommended )
      Est.Cy(i).cv=(Vyy-Vy'*inv(V1+1e-8*eye(V1))*Vy)/Est.ka(i);
    end
  end
  Est.Cp.th=fnorm(Est.Cp.V,2);               // pt.est. - pointer param.
  [xxx,ce(t)]=max(w);                        // pt.est. - pointer value

  // mixture prediction
  yy=zeros(my,1);
  for i=1:nc
    yy=yy+w(i)*Est.Cy(i).th';
  end
  yp(:,t)=yy;
end
Est.Cp.yt=ce;
Est.yp=yp;

// =========================== RESULTS
s=(nd-20):nd;
set(scf(1),'position',[100 100 700 600])
title('Evolution of parameter point estimates')
for i=1:nc
  subplot(nc,1,i)
  plot(Est.Cy(i).tht')
end

s=1:length(ce);
//q=1:nc; // 
[q,T]=c2c(ct(s),ce');
set(scf(2),'position',[700 100 800 600])
plot(q(ce(s)),':x')
plot(ct(s),'r:.','markersize',5)
title 'Class point estimates'

s=(fix(nd/2)+1):nd;
set(scf(3),'position',[750 200 700 600])
title('Comparison of prediction with the output')
for i=1:my
  subplot(nc,1,i)
  plot(s,yt(i,s),'b',s,yp(i,s),'r')
  legend('output','prediction');
end

save _data/dataT72.dat Est Sim
