function [p, p_full]=logit_inv(z,k)
// [p p_full]=logit_inv(z,k)    inversion to logit
// p       logistic parameter  (prob. for y=i, i=2,3,..,n)
// p_full  full logistic parameter   -"- for   i=1,2,3,..,n)
// z       logit .. = x.p
// k       choice for computation, k=1 - comp. according to definition
//                                 k=2 - comp. in logarithms

if argn(2)<2, k=1; end
if k==1
  // according to the definition
  p=zeros(z);
  for i=1:size(z,1)
    x=z(i,:);                           // one reg.vector
    p(i,:)=exp(x)/(1+sum(exp(x),1));      // inv logit for a single prob.
  end
  p_full=[1-sum(p,2) p];                // for prob(y=1)
else
  // in logarithms
  p=[zeros(size(z,1),1) z];             // log of p (all prob.)
  p=p-min(p,'c')*ones(1,size(p,2));       // normalization of logarithms
  p=exp(p);                             // taking an exponent
  p_full=p./(sum(p,2)*ones(1,size(p,2))); // normalization to sum=1
  p=p_full(:,2:$);                      // cutting probs for y=1
end
endfunction
